clear classes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the files:
%
% Longitude:
f1=input('Longitude coordinate filename: ','s');
t1=load(f1);
o1=ones(size(t1(:,1)));
s1=size(o1,1);
long=[t1(:,1),t1(:,2)/100];
eln=1e4*o1./(t1(:,3).^2);
% Remove the mean for greater solution stability:
long(:,2)=-sum(long(:,2).*eln)*o1/sum(eln)+long(:,2);
%
% Latitude:
f2=input('Latitude coordinate filename: ','s');
t2=load(f2);
o2=ones(size(t2(:,1)));
s2=size(o2,1);
lat=[t2(:,1),t2(:,2)/100];
elt=1e4*o2./(t2(:,3).^2);
% Remove the mean for greater solution stability:
lat(:,2)=-sum(lat(:,2).*elt)*o2/sum(elt)+lat(:,2);
%
% Height:
f3=input('Height coordinate filename: ','s');
t3=load(f3);
o3=ones(size(t3(:,1)));
s3=size(o3,1);
hgt=[t3(:,1),t3(:,2)/100];
eht=1e4*o3./(t3(:,3).^2);
% Remove the mean for greater solution stability:
hgt(:,2)=-sum(hgt(:,2).*eht)*o3/sum(eht)+hgt(:,2);
%
% Remove the mean time for greater solution stability:
Tm=(mean(long(:,1))+mean(lat(:,1))+mean(hgt(:,1)))/3;
long(:,1)=long(:,1)-Tm*o1;
lat(:,1)=lat(:,1)-Tm*o2;
hgt(:,1)=hgt(:,1)-Tm*o3;
%
% Subtract the angular velocity of NOAM plate:
f4=input('File containing NOAM plate correction: ','s');
v0=load(f4);
long(:,2)=long(:,2)-v0(1)*long(:,1);
lat(:,2)=lat(:,2)-v0(2)*lat(:,1);
hgt(:,2)=hgt(:,2)-v0(3)*hgt(:,1);
%
% Sparse variance matrices:
selt=sparse(diag(elt,0));
seln=sparse(diag(eln,0));
seht=sparse(diag(eht,0));
sall=sparse(diag([eln;elt;eht],0));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time axis delimitors:
j1=min([long(:,1);lat(:,1);hgt(:,1)]);
j2=max([long(:,1);lat(:,1);hgt(:,1)]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant velocity estimate:

disp('  ')
disp('  ')
disp('Constant Velocity Solution: ')
disp('  ')
% East
sz=size(long);
G=[long(:,1),o1];
per=inv(G'*seln*G);
p=per*G'*seln*long(:,2);
r=G*p-long(:,2);
rms=sqrt(sum(eln.*(r.^2))/sum(eln));
rmst=sum(eln.*(r.^2));
chisq=r'*seln*r/(sz(1)-length(p));
Vel_E=p(1);
p2E=p(2);
Del_E=2.*sqrt(chisq*per(1,1));
disp(sprintf('East Velocity: %6.2f +/- %6.2f cm/yr',Vel_E*100,Del_E*100))
disp(sprintf('RMS residual: %6.2f cm',rms*100))
disp('  ')

% North
sz=size(lat);
G=[lat(:,1),o2];
per=inv(G'*selt*G);
p=per*G'*selt*lat(:,2);
r=G*p-lat(:,2);
rms=sqrt(sum(elt.*(r.^2))/sum(elt));
rmst=rmst+sum(elt.*(r.^2));
chisq=r'*selt*r/(sz(1)-length(p));
Vel_N=p(1);
p2N=p(2);
Del_N=2.*sqrt(chisq*per(1,1));
disp(sprintf('North Velocity: %6.2f +/- %6.2f cm/yr',Vel_N*100,Del_N*100))
disp(sprintf('RMS residual: %6.2f cm',rms*100))
disp('  ')

% Up
sz=size(hgt);
G=[hgt(:,1),o3];
per=inv(G'*seht*G);
p=per*G'*seht*hgt(:,2);
r=G*p-hgt(:,2);
rms=sqrt(sum(eht.*(r.^2))/sum(eht));
rmst=rmst+sum(eht.*(r.^2));
chisq=r'*seht*r/(sz(1)-length(p));
Vel_U=p(1);
p2U=p(2);
Del_U=2.*sqrt(chisq*per(1,1));
disp(sprintf('Up Velocity: %6.2f +/- %6.2f cm/yr',Vel_U*100,Del_U*100))
disp(sprintf('RMS residual: %6.2f cm',rms*100))
disp('  ')
disp(sprintf('RMS residual (all data): %6.2f cm',sqrt(rmst/sum([eln;elt;eht]))*100))
disp('  ')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fit a time-varying model of the form:
% U=b+Vt+dd*(tanh((t-t_0)/tau)-1)/2

emin=1e20;
B=[p2E;p2N;p2U];
V=[Vel_E,Vel_N,Vel_U];
Dd=[0;0;0];
T_0=j1;
Tau=0;

% Grid search over t_0:
for i=0:1000,
  t_0=j1+i*(j2-j1)/1000;

% Initial model guess:
  tau=(j2-j1)/10;

  for iter=1:10,

% Linear portion:
    clear G d m
    G=zeros(s1+s2+s3,9);
    G(1:s1,1)=o1;
    G(s1+1:s1+s2,2)=o2;
    G(s1+s2+1:s1+s2+s3,3)=o3;
    G(1:s1,4)=long(:,1);
    G(s1+1:s1+s2,5)=lat(:,1);
    G(s1+s2+1:s1+s2+s3,6)=hgt(:,1);
    G(1:s1,7)=(tanh((long(:,1)-t_0*o1)/tau)-o1)/2;
    G(s1+1:s1+s2,8)=(tanh((lat(:,1)-t_0*o2)/tau)-o2)/2;
    G(s1+s2+1:s1+s2+s3,9)=(tanh((hgt(:,1)-t_0*o3)/tau)-o3)/2;
    d(1:s1)=long(:,2);
    d(s1+1:s1+s2)=lat(:,2);
    d(s1+s2+1:s1+s2+s3)=hgt(:,2);
    vars=G'*sall*G;
    m=vars\(G'*sall*d');

% Nonlinear portion:
    r(1:s1)=long(:,2)-m(1)*o1-m(4)*long(:,1)-m(7)*(tanh((long(:,1)-t_0*o1)/tau)-o1)/2;
    r(s1+1:s1+s2)=lat(:,2)-m(2)*o2-m(5)*lat(:,1)-m(8)*(tanh((lat(:,1)-t_0*o2)/tau)-o2)/2;
    r(s1+s2+1:s1+s2+s3)=hgt(:,2)-m(3)*o3-m(6)*hgt(:,1)-m(9)*(tanh((hgt(:,1)-t_0*o3)/tau)-o3)/2;
    clear G
    G=zeros(s1+s2+s3,1);
    G(1:s1,1)=-m(7)*(long(:,1)-t_0*o1).*(sech((long(:,1)-t_0*o1)/tau)).^2/(2*tau^2);
    G(s1+1:s1+s2,1)=-m(8)*(lat(:,1)-t_0*o2).*(sech((lat(:,1)-t_0*o2)/tau)).^2/(2*tau^2);
    G(s1+s2+1:s1+s2+s3,1)=-m(9)*(hgt(:,1)-t_0*o3).*(sech((hgt(:,1)-t_0*o3)/tau)).^2/(2*tau^2);
    dm=(G'*sall*G)\(G'*sall*r);
    tau=tau+dm';
    r(1:s1)=long(:,2)-m(1)*o1-m(4)*long(:,1)-m(7)*(tanh((long(:,1)-t_0*o1)/tau)-o1)/2;
    r(s1+1:s1+s2)=lat(:,2)-m(2)*o2-m(5)*lat(:,1)-m(8)*(tanh((lat(:,1)-t_0*o2)/tau)-o2)/2;
    r(s1+s2+1:s1+s2+s3)=hgt(:,2)-m(3)*o3-m(6)*hgt(:,1)-m(9)*(tanh((hgt(:,1)-t_0*o3)/tau)-o3)/2;
    rrr=sqrt(r'*sall*r/sum([eln;elt;eht]));
    if rrr < emin
      B=m(1:3);
      V=m(4:6);
      Dd=m(7:9);
      T_0=t_0;
      Tau=tau;
      emin=rrr;
      Vars=diag(inv(vars));
      chisq=(r'*sall*r)/(s1+s2+s3-11);
    end
  end
end

subplot(3,1,1)
a=plot(long(:,1)+Tm*o1,long(:,2),'.r');
set(a,'MarkerSize',15)
hold on
a=plot(long(:,1)+Tm*o1,B(1)*o1+V(1)*long(:,1)+Dd(1)*(tanh((long(:,1)-T_0*o1)/Tau)-o1)/2,'-b');
set(a,'LineWidth',2)
axis tight
subplot(3,1,2)
a=plot(lat(:,1)+Tm*o2,lat(:,2),'.r');
set(a,'MarkerSize',15)
hold on
a=plot(lat(:,1)+Tm*o2,B(2)*o2+V(2)*lat(:,1)+Dd(2)*(tanh((lat(:,1)-T_0*o2)/Tau)-o2)/2,'-b');
set(a,'LineWidth',2)
axis tight
subplot(3,1,3)
a=plot(hgt(:,1)+Tm*o3,hgt(:,2),'.r');
set(a,'MarkerSize',15)
hold on
a=plot(hgt(:,1)+Tm*o3,B(3)*o3+V(3)*hgt(:,1)+Dd(3)*(tanh((hgt(:,1)-T_0*o3)/Tau)-o3)/2,'-b');
set(a,'LineWidth',2)
axis tight

disp('  ')
disp('  ')
disp('Constant Velocity Superimposed By Hyperbolic Tangent: ')
disp('  ')
disp(sprintf('East Velocity: %6.2f +/- %6.2f cm/yr',V(1)*100,100*sqrt(Vars(4)*chisq)))
disp(sprintf('East Offset Displacement: %6.2f +/- %6.2f cm',Dd(1)*100,100*sqrt(Vars(7)*chisq)))
disp('  ')
disp(sprintf('North Velocity: %6.2f +/- %6.2f cm/yr',V(2)*100,100*sqrt(Vars(5)*chisq)))
disp(sprintf('North Offset Displacement: %6.2f +/- %6.2f cm',Dd(2)*100,100*sqrt(Vars(8)*chisq)))
disp('  ')
disp(sprintf('Up Velocity: %6.2f +/- %6.2f cm/yr',V(3)*100,100*sqrt(Vars(6)*chisq)))
disp(sprintf('Up Offset Displacement: %6.2f +/- %6.2f cm',Dd(3)*100,100*sqrt(Vars(9)*chisq)))
disp('  ')
disp(sprintf('Mid-Time of Displacement Transient: %9.3f',T_0+Tm))
disp(sprintf('Approximate Period of Transient: %6.3f Years',4*Tau))
disp('  ')
disp(sprintf('RMS residual: %6.2f cm',emin*100))
